@@ -1,32 +1,14 @@
 import asyncio
 import math
 import operator
-import os
-import unittest
+import re
 import time
-from textwrap import dedent
-
-try:
-    import numpy as np
-except ModuleNotFoundError:
-    if os.getenv('PARAM_TEST_NUMPY','0') == '1':
-        raise ImportError("PARAM_TEST_NUMPY=1 but numpy not available.")
-    else:
-        raise unittest.SkipTest("numpy not available")
-
-try:
-    import pandas as pd
-except ModuleNotFoundError:
-    if os.getenv('PARAM_TEST_PANDAS','0') == '1':
-        raise ImportError("PARAM_TEST_PANDAS=1 but pandas not available.")
-    else:
-        raise unittest.SkipTest("pandas not available")
 
 import param
 import pytest
 
-from param.parameterized import Skip, Parameter
-from param.reactive import bind, rx, reactive_ops
+from param.parameterized import Skip
+from param.reactive import bind, rx
 
 from .utils import async_wait_until
 
@@ -48,14 +30,17 @@ COMPARISON_OPERATORS = (
 
 LOGIC_UNARY_OPERATORS = (operator.inv,)
 
-NUMPY_UFUNCS = (np.min, np.max)
+NUMPY_UFUNCS = ("min", "max")
 
 @pytest.fixture(scope='module')
 def series():
+    np = pytest.importorskip("numpy")
+    pd = pytest.importorskip("pandas")
     return pd.Series(np.arange(5.0), name='A')
 
 @pytest.fixture(scope='module')
 def df():
+    pd = pytest.importorskip("pandas")
     return pd._testing.makeMixedDataFrame()
 
 class Parameters(param.Parameterized):
@@ -137,6 +122,7 @@ def test_reactive_getitem_list_with_slice():
     assert sx.rx.value == lst[i.rx.value: j.rx.value]
 
 def test_reactive_getitem_numpy_with_tuple():
+    np = pytest.importorskip("numpy")
     i = rx(0)
     j = rx(1)
     arr = np.array([[1, 2, 3], [4, 5, 6], [7, 8, 9]])
@@ -148,6 +134,8 @@ def test_reactive_getitem_numpy_with_tuple():
 
 @pytest.mark.parametrize('ufunc', NUMPY_UFUNCS)
 def test_numpy_ufunc(ufunc):
+    np = pytest.importorskip("numpy")
+    ufunc = getattr(np, ufunc)
     l = [1, 2, 3]
     assert ufunc(rx(l)).rx.value == ufunc(l)
     array = np.ndarray([1, 2, 3])
@@ -159,12 +147,18 @@ def test_reactive_empty_construct():
     i.rx.value = 2
     assert i.rx.value == 2
 
-def test_reactive_set_new_value():
+def test_reactive_set_new_value_assignment():
     i = rx(1)
     assert i.rx.value == 1
     i.rx.value = 2
     assert i.rx.value == 2
 
+def test_reactive_set_new_value_method():
+    i = rx(1)
+    assert i.rx.value == 1
+    i.rx.set(2)
+    assert i.rx.value == 2
+
 def test_reactive_increment_value():
     i = rx(1)
     assert i.rx.value == 1
@@ -274,14 +268,17 @@ def test_reactive_pipeline_reflect_bound_function():
     assert i.rx.value == 6
 
 def test_reactive_dataframe_method_chain(dataframe):
+    pd = pytest.importorskip("pandas")
     dfi = rx(dataframe).groupby('str')[['float']].mean().reset_index()
     pd.testing.assert_frame_equal(dfi.rx.value, dataframe.groupby('str')[['float']].mean().reset_index())
 
 def test_reactive_dataframe_attribute_chain(dataframe):
+    np = pytest.importorskip("numpy")
     array = rx(dataframe).str.values.rx.value
     np.testing.assert_array_equal(array, dataframe.str.values)
 
 def test_reactive_dataframe_param_value_method_chain(dataframe):
+    pd = pytest.importorskip("pd")
     P = Parameters(string='str')
     dfi = rx(dataframe).groupby(P.param.string)[['float']].mean().reset_index()
     pd.testing.assert_frame_equal(dfi.rx.value, dataframe.groupby('str')[['float']].mean().reset_index())
@@ -293,7 +290,7 @@ def test_reactive_len():
     l = i.rx.len()
     assert l.rx.value == 3
     i.rx.value = [1, 2]
-    assert l == 2
+    assert l.rx.value == 2
 
 def test_reactive_bool():
     i = rx(1)
@@ -584,8 +581,7 @@ async def test_reactive_gen():
 
     rxgen = rx(gen)
     assert rxgen.rx.value is param.Undefined
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 1
+    await async_wait_until(lambda: rxgen.rx.value == 1, interval=10, delay=0.0001)
     await async_wait_until(lambda: rxgen.rx.value == 2)
 
 async def test_reactive_gen_pipe():
@@ -598,12 +594,10 @@ async def test_reactive_gen_pipe():
     rxgen = rxv.rx.pipe(gen)
     rxgen.rx.watch()
     assert rxgen.rx.value is param.Undefined
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 1
+    await async_wait_until(lambda: rxgen.rx.value == 1, delay=0.04)
     await async_wait_until(lambda: rxgen.rx.value == 2)
     rxv.rx.value = 2
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 3
+    await async_wait_until(lambda: rxgen.rx.value == 3, delay=0.04)
     await async_wait_until(lambda: rxgen.rx.value == 4)
 
 async def test_reactive_gen_with_dep():
@@ -615,11 +609,9 @@ async def test_reactive_gen_with_dep():
     irx = rx(0)
     rxgen = rx(bind(gen, irx))
     assert rxgen.rx.value is param.Undefined
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 1
+    await async_wait_until(lambda: rxgen.rx.value == 1, delay=0.04)
     irx.rx.value = 3
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 4
+    await async_wait_until(lambda: rxgen.rx.value == 4, delay=0.04)
     await async_wait_until(lambda: rxgen.rx.value == 5)
 
 async def test_reactive_gen_pipe_with_dep():
@@ -633,15 +625,12 @@ async def test_reactive_gen_pipe_with_dep():
     rxgen = rxv.rx.pipe(bind(gen, irx))
     rxgen.rx.watch()
     assert rxgen.rx.value is param.Undefined
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 1
+    await async_wait_until(lambda: rxgen.rx.value == 1, delay=0.04)
     irx.rx.value = 3
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 4
+    await async_wait_until(lambda: rxgen.rx.value == 4, delay=0.04)
     await async_wait_until(lambda: rxgen.rx.value == 5)
     rxv.rx.value = 5
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 9
+    await async_wait_until(lambda: rxgen.rx.value == 9, delay=0.04)
     await async_wait_until(lambda: rxgen.rx.value == 10)
 
 async def test_reactive_async_gen():
@@ -652,8 +641,7 @@ async def test_reactive_async_gen():
 
     rxgen = rx(gen)
     assert rxgen.rx.value is param.Undefined
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 1
+    await async_wait_until(lambda: rxgen.rx.value == 1, delay=0.04)
     await async_wait_until(lambda: rxgen.rx.value == 2)
 
 async def test_reactive_async_gen_pipe():
@@ -664,8 +652,7 @@ async def test_reactive_async_gen_pipe():
 
     rxgen = rx(0).rx.pipe(gen)
     assert rxgen.rx.value is param.Undefined
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 1
+    await async_wait_until(lambda: rxgen.rx.value == 1, delay=0.04)
     await async_wait_until(lambda: rxgen.rx.value == 2)
 
 async def test_reactive_async_gen_with_dep():
@@ -677,13 +664,11 @@ async def test_reactive_async_gen_with_dep():
     irx = rx(0)
     rxgen = rx(bind(gen, irx))
     assert rxgen.rx.value is param.Undefined
-    await asyncio.sleep(0.05)
-    assert rxgen.rx.value == 1
+    await async_wait_until(lambda: rxgen.rx.value == 1, delay=0.05)
     irx.rx.value = 3
     await asyncio.sleep(0.05)
     irx.rx.value = 4
-    await asyncio.sleep(0.1)
-    assert rxgen.rx.value == 5
+    await async_wait_until(lambda: rxgen.rx.value == 5, delay=0.1)
 
 async def test_reactive_async_gen_pipe_with_dep():
     async def gen(value, i):
@@ -696,16 +681,13 @@ async def test_reactive_async_gen_pipe_with_dep():
     rxgen = rxv.rx.pipe(bind(gen, i=irx))
     rxgen.rx.watch()
     assert rxgen.rx.value is param.Undefined
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 1
+    await async_wait_until(lambda: rxgen.rx.value == 1, delay=0.04)
     irx.rx.value = 3
     await asyncio.sleep(0.04)
     irx.rx.value = 4
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 5
+    await async_wait_until(lambda: rxgen.rx.value == 5, delay=0.04)
     rxv.rx.value = 5
-    await asyncio.sleep(0.04)
-    assert rxgen.rx.value == 10
+    await async_wait_until(lambda: rxgen.rx.value == 10, delay=0.04)
     await async_wait_until(lambda: rxgen.rx.value == 11)
 
 def test_root_invalidation():
@@ -764,12 +746,31 @@ def test_ensure_ref_can_update_by_watcher_of_same_parameter():
     assert t.lst[1] == "TEST"
 
 def test_reactive_callback_resolve_accessor():
+    pd = pytest.importorskip("pandas")
     df = pd.DataFrame({"name": ["Bill", "Bob"]})
     dfx = rx(df)
     out = dfx["name"].str._callback()
     assert out is df["name"].str
 
-def test_docstrings_in_sync():
-    # The docstring needs to be explicitly written to work with LSP.
-    assert dedent(reactive_ops.__doc__) == dedent(Parameter.rx.__doc__)
-    assert dedent(reactive_ops.__doc__) == dedent(rx.rx.__doc__)
+
+def test_reactive_dunder_len_error():
+    with pytest.raises(
+        TypeError,
+        match=re.escape(
+            'len(<rx_obj>) is not supported. Use `<rx_obj>.rx.len()` to '
+            'obtain the length as a reactive expression, or '
+            '`len(<rx_obj>.rx.value)` to obtain the length of the underlying '
+            'expression value.'
+        )
+    ):
+        len(rx([1, 2]))
+
+
+def test_reactive_dunder_bool():
+    assert bool(rx([1, 2]))
+
+
+def test_reactive_set_value_attributeerror():
+    x = rx(1)
+    with pytest.raises(AttributeError, match="'rx' has no attribute 'value'"):
+        x.value = 1
