@@ -35,7 +35,7 @@ class MockLoggingHandler(logging.Handler):
         super().__init__(*args, **kwargs)
 
     def emit(self, record):
-        """Store a message to the instance's messages dictionary"""
+        """Store a message to the instance's messages dictionary."""
         self.acquire()
         try:
             self.messages[record.levelname].append(record.getMessage())
@@ -49,7 +49,7 @@ class MockLoggingHandler(logging.Handler):
         self.release()
 
     def tail(self, level, n=1):
-        """Returns the last n lines captured at the given level"""
+        """Return the last n lines captured at the given level."""
         return [str(el) for el in self.messages[level][-n:]]
 
     def assertEndsWith(self, level, substring):
@@ -116,7 +116,7 @@ def warnings_as_excepts(match=None):
         param.parameterized.warnings_as_exceptions = orig
 
 
-async def async_wait_until(fn, timeout=5000, interval=100):
+async def async_wait_until(fn, timeout=5000, interval=100, delay=0):
     """
     Exercise a test function in a loop until it evaluates to True
     or times out.
@@ -137,6 +137,8 @@ async def async_wait_until(fn, timeout=5000, interval=100):
         Total timeout in milliseconds, by default 5000
     interval : int, optional
         Waiting interval, by default 100
+    delay : int, optional
+        Delay before starting, by default 0
 
     Adapted from pytest-qt.
 
@@ -144,6 +146,9 @@ async def async_wait_until(fn, timeout=5000, interval=100):
     # Hide this function traceback from the pytest output if the test fails
     __tracebackhide__ = True
 
+    if delay:
+        await asyncio.sleep(delay)
+
     start = time.monotonic()
 
     def timed_out():
